#!/usr/bin/env python

import platform
import sys

import json

from charmhelpers.contrib.openstack.utils import configure_installation_source

from charmhelpers.core.hookenv import (
    CRITICAL,
    Hooks,
    UnregisteredHookError,
    config,
    local_unit,
    log,
    relation_get,
    relation_set,
    remote_unit
)

from charmhelpers.core.host import (
    pwgen,
    restart_on_change,
    service_restart
)

from charmhelpers.fetch import (
    apt_install,
    apt_upgrade,
    configure_sources
)

from contrail_configuration_utils import (
    fix_permissions,
    write_contrail_api_config,
    write_contrail_schema_config,
    write_discovery_config,
    write_ifmap_config,
    write_vnc_api_config
)

PACKAGES = [ "ifmap-server", "contrail-config", "contrail-config-openstack",
             "neutron-common" ]

hooks = Hooks()

@hooks.hook("amqp-relation-changed")
def amqp_changed():
    if not relation_get("password"):
        log("Relation not ready")
        return
    amqp_relation()

@hooks.hook("amqp-relation-departed")
@hooks.hook("amqp-relation-broken")
@restart_on_change({"/etc/contrail/contrail-api.conf": ["contrail-api"]})
def amqp_relation():
    write_contrail_api_config()

@hooks.hook("amqp-relation-joined")
def amqp_joined():
    relation_set(username="contrail", vhost="contrail")

@hooks.hook("cassandra-relation-changed")
def cassandra_changed():
    if not relation_get("port"):
        log("Relation not ready")
        return
    cassandra_relation()

@hooks.hook("cassandra-relation-departed")
@hooks.hook("cassandra-relation-broken")
@restart_on_change({"/etc/contrail/contrail-api.conf": ["contrail-api"],
                    "/etc/contrail/contrail-schema.conf": ["contrail-schema"],
                    "/etc/contrail/discovery.conf": ["contrail-discovery"]})
def cassandra_relation():
    write_contrail_api_config()
    write_contrail_schema_config()
    write_discovery_config()

@hooks.hook("config-changed")
def config_changed():
    pass

@hooks.hook("contrail-api-relation-joined")
def contrail_api_joined():
    relation_set(port=8082)

@hooks.hook("contrail-ifmap-relation-broken")
@restart_on_change({"/etc/ifmap-server/basicauthusers.properties": ["ifmap-server"]})
def contrail_ifmap_broken():
    write_ifmap_config()

@hooks.hook("contrail-ifmap-relation-departed")
def contrail_ifmap_departed():
    creds = relation_get("creds", local_unit())
    creds = json.loads(creds) if creds else {}
    unit = remote_unit()
    if unit not in creds:
        return
    del creds[unit]
    relation_set(creds=json.dumps(creds))
    write_ifmap_config()
    service_restart("ifmap-server")

@hooks.hook("contrail-ifmap-relation-joined")
def contrail_ifmap_joined():
    creds = relation_get("creds", local_unit())
    creds = json.loads(creds) if creds else {}
    unit = remote_unit()
    if unit in creds:
        return
    creds[unit] = { "username": unit, "password": pwgen() }
    relation_set(creds=json.dumps(creds))
    write_ifmap_config()
    service_restart("ifmap-server")

@hooks.hook("identity-admin-relation-changed")
def identity_admin_changed():
    if not relation_get("service_hostname"):
        log("Relation not ready")
        return
    identity_admin_relation()

@hooks.hook("identity-admin-relation-departed")
@hooks.hook("identity-admin-relation-broken")
@restart_on_change({"/etc/contrail/contrail-api.conf": ["contrail-api"],
                    "/etc/contrail/contrail-schema.conf": ["contrail-schema"],
                    "/etc/contrail/vnc_api_lib.ini": []})
def identity_admin_relation():
    write_contrail_api_config()
    write_contrail_schema_config()
    write_vnc_api_config()

@hooks.hook()
def install():
    if platform.linux_distribution()[2] != "precise":
        log("Charm only supports Precise currently", CRITICAL)
        sys.exit(1)
    configure_installation_source(config("openstack-origin"))
    configure_sources(True, "install-sources")
    apt_upgrade(fatal=True, dist=True)
    apt_install(PACKAGES, fatal=True)
    fix_permissions()

def main():
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log("Unknown hook {} - skipping.".format(e))

@hooks.hook("upgrade-charm")
def upgrade_charm():
    write_ifmap_config()
    write_contrail_api_config()
    write_contrail_schema_config()
    write_discovery_config()
    for service in [ "ifmap-server", "contrail-api", "contrail-schema",
                     "contrail-discovery" ]:
        service_restart(service)

@hooks.hook("zookeeper-relation-changed")
def zookeeper_changed():
    if not relation_get("port"):
        log("Relation not ready")
        return
    zookeeper_relation()

@hooks.hook("zookeeper-relation-departed")
@hooks.hook("zookeeper-relation-broken")
@restart_on_change({"/etc/contrail/contrail-api.conf": ["contrail-api"],
                    "/etc/contrail/contrail-schema.conf": ["contrail-schema"],
                    "/etc/contrail/discovery.conf": ["contrail-discovery"]})
def zookeeper_relation():
    write_contrail_api_config()
    write_contrail_schema_config()
    write_discovery_config()

if __name__ == "__main__":
    main()
